#!/usr/bin/env python
# -*- coding:utf-8 -*-
import re
from types import ListType

from docutils import nodes


class BaseTranslator(nodes.NodeVisitor):
    """
    """
    def __init__(self, document):
        nodes.NodeVisitor.__init__(self, document)
        self.body = []

    def astext(self):
        return ''.join(self.body)

    def dispatch_visit(self, node):
        #print node.__class__.__name__
        return nodes.NodeVisitor.dispatch_visit(self, node)

    def visit_Text(self, node):
        self.body.append(node.astext())

    def depart_Text(self, node):
        pass

    # ==============
    #  Root Element
    # ==============

    def visit_document(self, node):
        pass

    def depart_document(self, node):
        pass

    # ================
    #  Title Elements
    # ================

    def visit_title(self, node):
        pass

    def depart_title(self, node):
        pass

    def visit_subtitle(self, node):
        pass

    def depart_subtitle(self, node):
        pass

    def visit_rubric(self, node):
        pass

    def depart_rubric(self, node):
        pass

    # ========================
    #  Bibliographic Elements
    # ========================
    def visit_docinfo(self, node):
        pass

    def depart_docinfo(self, node):
        pass

    def visit_author(self, node):
        pass

    def depart_author(self, node):
        pass

    def visit_authors(self, node):
        pass

    def depart_authors(self, node):
        pass

    def visit_organization(self, node):
        pass

    def depart_organization(self, node):
        pass

    def visit_address(self, node):
        pass

    def depart_address(self, node):
        pass

    def visit_contact(self, node):
        pass

    def depart_contact(self, node):
        pass

    def visit_version(self, node):
        pass

    def depart_version(self, node):
        pass

    def visit_revision(self, node):
        pass

    def depart_revision(self, node):
        pass

    def visit_status(self, node):
        pass

    def depart_status(self, node):
        pass

    def visit_date(self, node):
        pass

    def depart_date(self, node):
        pass

    def visit_copyright(self, node):
        pass

    def depart_copyright(self, node):
        pass

    # =====================
    #  Decorative Elements
    # =====================

    def visit_decoration(self, node):
        pass

    def depart_decoration(self, node):
        pass

    def visit_header(self, node):
        pass

    def depart_header(self, node):
        pass

    def visit_footer(self, node):
        pass

    def depart_footer(self, node):
        pass

    # =====================
    #  Structural Elements
    # =====================

    def visit_section(self, node):
        pass

    def depart_section(self, node):
        pass

    def visit_topic(self, node):
        pass

    def depart_topic(self, node):
        pass

    def visit_sidebar(self, node):
        pass

    def depart_sidebar(self, node):
        pass

    def visit_transition(self, node):
        pass

    def depart_transition(self, node):
        pass

    # ===============
    #  Body Elements
    # ===============

    def visit_paragraph(self, node):
        pass

    def depart_paragraph(self, node):
        pass

    def visit_compound(self, node):
        pass

    def depart_compound(self, node):
        pass

    def visit_container(self, node):
        pass

    def depart_container(self, node):
        pass

    def visit_bullet_list(self, node):
        pass

    def depart_bullet_list(self, node):
        pass

    def visit_enumerated_list(self, node):
        pass

    def depart_enumerated_list(self, node):
        pass

    def visit_list_item(self, node):
        pass

    def depart_list_item(self, node):
        pass

    def visit_definition_list(self, node):
        pass

    def depart_definition_list(self, node):
        pass

    def visit_definition_list_item(self, node):
        pass

    def depart_definition_list_item(self, node):
        pass

    def visit_term(self, node):
        pass

    def depart_term(self, node):
        pass

    def visit_classifier(self, node):
        pass

    def depart_classifier(self, node):
        pass

    def visit_definition(self, node):
        pass

    def depart_definition(self, node):
        pass

    def visit_field_list(self, node):
        pass

    def depart_field_list(self, node):
        pass

    def visit_field(self, node):
        pass

    def depart_field(self, node):
        pass

    def visit_field_name(self, node):
        pass

    def depart_field_name(self, node):
        pass

    def visit_field_body(self, node):
        pass

    def depart_field_body(self, node):
        pass

    def visit_option(self, node):
        pass

    def depart_option(self, node):
        pass

    def visit_option_argument(self, node):
        pass

    def depart_option_argument(self, node):
        pass

    def visit_option_group(self, node):
        pass

    def depart_option_group(self, node):
        pass

    def visit_option_list(self, node):
        pass

    def depart_option_list(self, node):
        pass

    def visit_option_list_item(self, node):
        pass

    def depart_option_list_item(self, node):
        pass

    def visit_option_string(self, node):
        pass

    def depart_option_string(self, node):
        pass

    def visit_description(self, node):
        pass

    def depart_description(self, node):
        pass

    def visit_literal_block(self, node):
        pass

    def depart_literal_block(self, node):
        pass

    def visit_doctest_block(self, node):
        pass

    def depart_doctest_block(self, node):
        pass

    def visit_line_block(self, node):
        pass

    def depart_line_block(self, node):
        pass

    def visit_line(self, node):
        pass

    def depart_line(self, node):
        pass

    def visit_block_quote(self, node):
        pass

    def depart_block_quote(self, node):
        pass

    def visit_attribution(self, node):
        pass

    def depart_attribution(self, node):
        pass

    def visit_attention(self, node):
        pass

    def depart_attention(self, node):
        pass

    def visit_caution(self, node):
        pass

    def depart_caution(self, node):
        pass

    def visit_danger(self, node):
        pass

    def depart_danger(self, node):
        pass

    def visit_error(self, node):
        pass

    def depart_error(self, node):
        pass

    def visit_important(self, node):
        pass

    def depart_important(self, node):
        pass

    def visit_note(self, node):
        pass

    def depart_note(self, node):
        pass

    def visit_tip(self, node):
        pass

    def depart_tip(self, node):
        pass

    def visit_hint(self, node):
        pass

    def depart_hint(self, node):
        pass

    def visit_warning(self, node):
        pass

    def depart_warning(self, node):
        pass

    def visit_admonition(self, node):
        pass

    def depart_admonition(self, node):
        pass

    def visit_comment(self, node):
        pass

    def depart_comment(self, node):
        pass

    def visit_substitution_definition(self, node):
        pass

    def depart_substitution_definition(self, node):
        pass

    def visit_target(self, node):
        pass

    def depart_target(self, node):
        pass

    def visit_footnote(self, node):
        pass

    def depart_footnote(self, node):
        pass

    def visit_citation(self, node):
        pass

    def depart_citation(self, node):
        pass

    def visit_label(self, node):
        pass

    def depart_label(self, node):
        pass

    def visit_figure(self, node):
        pass

    def depart_figure(self, node):
        pass

    def visit_caption(self, node):
        pass

    def depart_caption(self, node):
        pass

    def visit_legend(self, node):
        pass

    def depart_legend(self, node):
        pass

    def visit_table(self, node):
        pass

    def depart_table(self, node):
        pass

    def visit_tgroup(self, node):
        pass

    def depart_tgroup(self, node):
        pass

    def visit_colspec(self, node):
        pass

    def depart_colspec(self, node):
        pass

    def visit_thead(self, node):
        pass

    def depart_thead(self, node):
        pass

    def visit_tbody(self, node):
        pass

    def depart_tbody(self, node):
        pass

    def visit_row(self, node):
        pass

    def depart_row(self, node):
        pass

    def visit_entry(self, node):
        pass

    def depart_entry(self, node):
        pass

    def visit_system_message(self, node):
        pass

    def depart_system_message(self, node):
        pass

    def visit_pending(self, node):
        pass

    def depart_pending(self, node):
        pass

    def visit_raw(self, node):
        pass

    def depart_raw(self, node):
        pass

    # =================
    #  Inline Elements
    # =================

    def visit_emphasis(self, node):
        pass

    def depart_emphasis(self, node):
        pass

    def visit_strong(self, node):
        pass

    def depart_strong(self, node):
        pass

    def visit_literal(self, node):
        pass

    def depart_literal(self, node):
        pass

    def visit_reference(self, node):
        pass

    def depart_reference(self, node):
        pass

    def visit_footnote_reference(self, node):
        pass

    def depart_footnote_reference(self, node):
        pass

    def visit_citation_reference(self, node):
        pass

    def depart_citation_reference(self, node):
        pass

    def visit_substitution_reference(self, node):
        pass

    def depart_substitution_reference(self, node):
        pass

    def visit_title_reference(self, node):
        pass

    def depart_title_reference(self, node):
        pass

    def visit_abbreviation(self, node):
        pass

    def depart_abbreviation(self, node):
        pass

    def visit_acronym(self, node):
        pass

    def depart_acronym(self, node):
        pass

    def visit_superscript(self, node):
        pass

    def depart_superscript(self, node):
        pass

    def visit_subscript(self, node):
        pass

    def depart_subscript(self, node):
        pass

    def visit_image(self, node):
        pass

    def depart_image(self, node):
        pass

    def visit_inline(self, node):
        pass

    def depart_inline(self, node):
        pass

    def visit_problematic(self, node):
        pass

    def depart_problematic(self, node):
        pass

    def visit_generated(self, node):
        pass

    def depart_generated(self, node):
        pass

    # ==================
    #  Hatena directives
    # ==================
    def visit_niconico(self, node):
        raise NotImplementedError('"niconico" directive is not implemented')

    def depart_niconico(self, node):
        raise NotImplementedError('"niconico" directive is not implemented')

    def visit_google(self, node):
        raise NotImplementedError('"google" directive is not implemented')

    def depart_google(self, node):
        raise NotImplementedError('"google" directive is not implemented')

    def visit_amazon(self, node):
        raise NotImplementedError('"amazon" directive is not implemented')

    def depart_amazon(self, node):
        raise NotImplementedError('"amazon" directive is not implemented')

    def visit_wikipedia(self, node):
        raise NotImplementedError('"wikipedia" directive is not implemented')

    def depart_wikipedia(self, node):
        raise NotImplementedError('"wikipedia" directive is not implemented')

    def visit_twitter(self, node):
        raise NotImplementedError('"twitter" directive is not implemented')

    def depart_twitter(self, node):
        raise NotImplementedError('"twitter" directive is not implemented')

    def visit_map(self, node):
        raise NotImplementedError('"map" directive is not implemented')

    def depart_map(self, node):
        raise NotImplementedError('"map" directive is not implemented')



# -*- coding:utf-8 -*-

# Copyright (c) 2011 Naoya INADA <naoina@kuune.org>
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.

__author__ = "Naoya INADA <naoina@kuune.org>"

import itertools

from docutils.nodes import General
from docutils.nodes import FixedTextElement
from docutils.parsers.rst import Directive
from docutils.parsers.rst import directives
from docutils.parsers.rst.directives import register_directive

from rst2hatena.directives.autolink import *

class code_block(General, FixedTextElement):
    pass


class CodeBlock(Directive):
    has_content = True
    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = {}

    def run(self):
        self.options['type'] = directives.unchanged_required(self.arguments[0])
        code = u'\n'.join(self.content)
        node = code_block(code, code, **self.options)
        return [node]

_rst2hatena_directives = (
    ('code-block', CodeBlock),)

_hatena_directives = (
    ('niconico', Niconico),
    ('google', Google),
    ('amazon', Amazon),
    ('wikipedia', Wikipedia),
    ('twitter', Twitter),
    ('map', Map))

for name,directive in itertools.chain(_rst2hatena_directives,
        _hatena_directives):
    register_directive(name, directive)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
A minimal front end to the Docutils Publisher, producing Hatena.
"""

try:
    import locale
    locale.setlocale(locale.LC_ALL, '')
except:
    pass

from docutils.core import publish_cmdline, default_description

import hatena_writer

description = ('Generates Hatena documents from standalone reStructuredText '
               'sources.  ' + default_description)


def main():
    publish_cmdline(writer=hatena_writer.Writer(), description=description)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding:utf-8 -*-
import unittest
from docutils.core import publish_file, publish_string

from rst2hatena.hatena_writer import Writer


class WriterTest(unittest.TestCase):
    """
    """
    def setUp(self):
        self.writer = Writer()

    def test_all(self):
        #output = publish_file(source_path='./sample.txt', writer=self.writer)
        #print '===================='
        #print output
        #print '===================='
        self.assert_(True)

    def test_emphasis(self):
        str = """\
emphasis
--------

*emphasis*
  Normally rendered as italics.
"""
        exp_str = """\
*emphasis

:<span style="font-style:italic;">emphasis</span>:Normally rendered as italics.

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)

    def test_strong_emphasis(self):
        str = """\
strong emphasis
---------------

**strong emphasis**
  Normally rendered as boldface.
"""
        exp_str = """\
*strong emphasis

:<span style="font-weight:bold;">strong emphasis</span>:Normally rendered as boldface.

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)

    def test_interpreted_text(self):
        str = """\
interpreted text
----------------

`interpreted text`
  The rendering and meaning of interpreted text is domain- or application-dependent. It can be used for things like index entries or explicit descriptive markup (like program identifiers).
"""
        exp_str = """\
*interpreted text

:<span style="font-style:italic;">interpreted text</span>:The rendering and meaning of interpreted text is domain- or application-dependent. It can be used for things like index entries or explicit descriptive markup (like program identifiers).

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)

    def test_inline_literal(self):
        str = """\
inline literal
--------------

``inline literal``
  Normally rendered as monospaced text. Spaces should be preserved, but line breaks will not be.
"""
        exp_str = """\
*inline literal

:<span style="font-style:italic;">inline literal</span>:Normally rendered as monospaced text. Spaces should be preserved, but line breaks will not be.

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)

    def test_no_reference(self):
        str = """\
reference
---------

reference_
  A simple, one-word hyperlink reference. See Hyperlink Targets.
"""
        exp_str = """\
*reference

:<a href="#reference">reference</a>:A simple, one-word hyperlink reference. See Hyperlink Targets.

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_no_phrase_reference(self):
        str = """\
phrase reference
----------------

`phrase reference`_
  A hyperlink reference with spaces or punctuation needs to be quoted with backquotes. See Hyperlink Targets.
"""
        exp_str = """\
*phrase reference

:<a href="#phrase-reference">phrase reference</a>:A hyperlink reference with spaces or punctuation needs to be quoted with backquotes. See Hyperlink Targets.

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_no_inline_internal_target(self):
        str = """\
inline internal target
----------------------

_`inline internal target`
  A crossreference target within text. See Hyperlink Targets.
"""
        exp_str = """\
*inline internal target

<!--
Duplicate implicit target name: "inline internal target".
-->

:inline internal target:A crossreference target within text. See Hyperlink Targets.

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_standalone_hyperlink(self):
        str = """\
standalone hyperlink
--------------------

http://docutils.sf.net/
  A standalone hyperlink.
"""
        exp_str = """\
*standalone hyperlink

:[http://docutils.sf.net/:title=http://docutils.sf.net/]:A standalone hyperlink.

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_section(self):
        str = """\
Section
=======

Subtitle
--------

Titles are underlined (or over- 
and underlined) with a printing 
nonalphanumeric 7-bit ASCII 
character. Recommended choices 
are "``= - ` : ' " ~ ^ _ * + # < >``". 
The underline/overline must be at 
least as long as the title text. 

A lone top-level (sub)section 
is lifted up to be the document's 
(sub)title.
"""
        exp_str = """\
*Section

**Subtitle

Titles are underlined (or over- and underlined) with a printing nonalphanumeric 7-bit ASCII character. Recommended choices are "<span style="font-style:italic;">= - ` : ' " ~ ^ _ * + # < ></span>". The underline/overline must be at least as long as the title text.

A lone top-level (sub)section is lifted up to be the document's (sub)title.

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_paragraphs(self):
        str = """\
Paragraphs
==========

This is a paragraph.

Paragraphs line up at their left 
edges, and are normally separated 
by blank lines.
"""
        exp_str = """\
*Paragraphs

This is a paragraph.

Paragraphs line up at their left edges, and are normally separated by blank lines.

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_bullet_lists(self):
        str = """\
Bullet Lists
============

Bullet lists:

- This is item 1 
- This is item 2

- Bullets are "-", "*" or "+". 
  Continuing text must be aligned 
  after the bullet and whitespace.

Note that a blank line is required 
before the first item and after the 
last, but is optional between items.
"""
        exp_str = """\
*Bullet Lists

Bullet lists:

- This is item 1
- This is item 2
- Bullets are "-", "*" or "+". Continuing text must be aligned after the bullet and whitespace.

Note that a blank line is required before the first item and after the last, but is optional between items.

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_enumerated_lists(self):
        str = """\
Enumerated Lists
================

Enumerated lists:

3. This is the first item 
4. This is the second item 
5. Enumerators are arabic numbers, 
   single letters, or roman numerals 
6. List items should be sequentially 
   numbered, but need not start at 1 
   (although not all formatters will 
   honour the first index). 
#. This item is auto-enumerated
"""
        exp_str = """\
*Enumerated Lists

Enumerated lists:

+ This is the first item
+ This is the second item
+ Enumerators are arabic numbers, single letters, or roman numerals
+ List items should be sequentially numbered, but need not start at 1 (although not all formatters will honour the first index).
+ This item is auto-enumerated

<!--
Enumerated list start value not ordinal-1: "3" (ordinal 3)
-->

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_definition_lists(self):
        str = """\
Definition Lists
================

Definition lists: 

what 
  Definition lists associate a term with 
  a definition. 

how 
  The term is a one-line phrase, and the 
  definition is one or more paragraphs or 
  body elements, indented relative to the 
  term. Blank lines are not allowed 
  between term and definition.
"""
        exp_str = """\
*Definition Lists

Definition lists:

:what:Definition lists associate a term with a definition.

:how:The term is a one-line phrase, and the definition is one or more paragraphs or body elements, indented relative to the term. Blank lines are not allowed between term and definition.

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)

    def test_field_lists(self):
        str = """\
Field Lists
===========

Fields
------

:Author: Yosuke Ikeda
:Authors: 
    Tony J. (Tibs) Ibbs, 
    David Goodger
    (and sundry other good-natured folks)

:Organization: Electro Insanity
:Address: Tokyo, Japan
:Contact: alpha.echo.35@gmail.com
:Status: normal
:Date: 2001/08/08
:Copyright: Yosuke Ikeda
:Version: 1.0 of 2001/08/08
:Revision: 12345
:Dedication: To my father.
"""
        exp_str = """\
*Field Lists

**Fields

:Author:Yosuke Ikeda
:Author:Tony J. (Tibs) Ibbs
:Author:David Goodger (and sundry other good-natured folks)
:Organization:Electro Insanity
:Address:Tokyo, Japan
:Contact:[mailto:alpha.echo.35@gmail.com:title=alpha.echo.35@gmail.com]
:Status:normal
:Date:2001/08/08
:Copyright:Yosuke Ikeda
:Version:1.0 of 2001/08/08
:Revision:12345
:Dedication:To my father.
"""
        output = publish_string(str, writer=self.writer)

        print
        self.assertEqual(exp_str, output)


    def test_option_lists(self):
        str = """\
Option Lists
============

-a            command-line option "a" 
-b file       options can have arguments 
              and long descriptions 
--long        options can be long also 
--input=file  long options can also have 
              arguments 
/V            DOS/VMS-style options too
"""
        exp_str = """\
*Option Lists

:-a:command-line option "a"

:-b file:options can have arguments and long descriptions

:--long:options can be long also

:--input=file:long options can also have arguments

:/V:DOS/VMS-style options too

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_literal_blocks(self):
        str = """\
Literal Blocks
==============

A paragraph containing only two colons 
indicates that the following indented 
or quoted text is a literal block. 

:: 

  The paragraph containing only '::' 
  will be omitted from the result. 

The ``::`` may be tacked onto the very 
end of any paragraph. The ``::`` will be 
omitted if it is preceded by whitespace. 
The ``::`` will be converted to a single 
colon if preceded by text, like this:: 

  It's very convenient to use this form. 

Per-line quoting can also be used on 
unindented literal blocks:: 

> Useful for quotes from email and 
> for Haskell literate programming.
"""
        exp_str = """\
*Literal Blocks

A paragraph containing only two colons indicates that the following indented or quoted text is a literal block.

>||
The paragraph containing only '::'
will be omitted from the result.
||<

The <span style="font-style:italic;">::</span> may be tacked onto the very end of any paragraph. The <span style="font-style:italic;">::</span> will be omitted if it is preceded by whitespace. The <span style="font-style:italic;">::</span> will be converted to a single colon if preceded by text, like this:

>||
It's very convenient to use this form.
||<

Per-line quoting can also be used on unindented literal blocks:

>||
> Useful for quotes from email and
> for Haskell literate programming.
||<

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_line_blocks(self):
        str = """\
Line Blocks
===========

| Line blocks are useful for addresses, 
| verse, and adornment-free lists. 
| 
| Each new line begins with a 
| vertical bar ("|"). 
|     Line breaks and initial indents 
|     are preserved. 
| Continuation lines are wrapped 
  portions of long lines; they begin 
  with spaces in place of vertical bars.
"""
        exp_str = """\
*Line Blocks

>|
Line blocks are useful for addresses,
verse, and adornment-free lists.

Each new line begins with a
vertical bar ("|").
    Line breaks and initial indents
    are preserved.
Continuation lines are wrapped portions of long lines; they begin with spaces in place of vertical bars.
|<

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_block_quotes(self):
        str = """\
Block Quotes
============

Block quotes are just:

    Indented paragraphs,

        and they may nest.
        and they may nest.
"""
        exp_str = """\
*Block Quotes

Block quotes are just:

>>
Indented paragraphs,

    and they may nest. and they may nest.
<<

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_doctest_blocks(self):
        str = """\
Doctest Blocks
==============

Doctest blocks are interactive 
Python sessions. They begin with 
"``>>>``" and end with a blank line.

>>> print "This is a doctest block." 
This is a doctest block.
"""
        exp_str = """\
*Doctest Blocks

Doctest blocks are interactive Python sessions. They begin with "<span style="font-style:italic;">>>></span>" and end with a blank line.

>|python|
>>> print "This is a doctest block."
This is a doctest block.
||<

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_grid_tables(self):
        str = """\
Grid table
----------

+------------+------------+-----------+
| Header 1   | Header 2   | Header 3  |
+============+============+===========+
| body row 1 | column 2   | column 3  |
+------------+------------+-----------+
| body row 2 | Cells may span columns.|
+------------+------------+-----------+
| body row 3 | Cells may  | - Cells   |
+------------+ span rows. | - contain |
| body row 4 |            | - blocks. |
+------------+------------+-----------+
"""
        exp_str = """\
*Grid table

|*Header 1|*Header 2|*Header 3|
|body row 1|column 2|column 3|
|body row 2|Cells may span columns.|
|body row 3|Cells may span rows.|- Cells
- contain
- blocks.
|
|body row 4|

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_simple_table(self):
        str = """\
Simple table
------------

=====  =====  ====== 
   Inputs     Output 
------------  ------ 
  A      B    A or B 
=====  =====  ====== 
False  False  False 
True   False  True 
False  True   True 
True   True   True 
=====  =====  ======
"""
        exp_str = """\
*Simple table

|*Inputs|*Output|
|*A|*B|*A or B|
|False|False|False|
|True|False|True|
|False|True|True|
|True|True|True|

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_transitions(self):
        str = """\
Transitions
===========

A transition marker is a horizontal line 
of 4 or more repeated punctuation 
characters.

------------

A transition should not begin or end a 
section or document, nor should two 
transitions be immediately adjacent.
"""
        exp_str = """\
*Transitions

A transition marker is a horizontal line of 4 or more repeated punctuation characters.

<hr>

A transition should not begin or end a section or document, nor should two transitions be immediately adjacent.

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_explicit_footnote_reference(self):
        str = """\
Footnote references
-------------------

Footnote references, like [5]_. 
Note that footnotes may get 
rearranged, e.g., to the bottom of 
the "page". [5]_

.. [5] A numerical footnote. Note 
   there's no colon after the ``]``.
"""
        exp_str = """\
*Footnote references

Footnote references, like ((A numerical footnote. Note there's no colon after the <span style="font-style:italic;">]</span>.)). Note that footnotes may get rearranged, e.g., to the bottom of the "page". ((A numerical footnote. Note there's no colon after the <span style="font-style:italic;">]</span>.))

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_explicit_markup_autonumbered_footnotes(self):
        str = """\
Autonumbered footnotes are 
possible, like using [#]_ and [#]_.

.. [#] This is the first one. 
.. [#] This is the second one.

They may be assigned 'autonumber 
labels' - for instance, 
[#fourth]_ and [#third]_.

.. [#third] a.k.a. third_

.. [#fourth] a.k.a. fourth_
"""
        exp_str = """\
Autonumbered footnotes are possible, like using ((This is the first one.)) and ((This is the second one.)).

They may be assigned 'autonumber labels' - for instance, ((a.k.a. <a href="#fourth">fourth</a>)) and ((a.k.a. <a href="#third">third</a>)).

"""

        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_explicit_markup_auto_symbol_footnotes(self):
        str = """\
Auto-symbol footnotes are also 
possible, like this: [*]_ and [*]_.

.. [*] This is the first one. 

.. [*] This is the second one.
"""
        exp_str = """\
Auto-symbol footnotes are also possible, like this: ((This is the first one.)) and ((This is the second one.)).

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_citations(self):
        str = """\
Citation references, like [CIT2002]_. 
Note that citations may get 
rearranged, e.g., to the bottom of 
the "page".

.. [CIT2002] A citation 
   (as often used in journals).

Citation labels contain alphanumerics, 
underlines, hyphens and fullstops. 
Case is not significant.

Given a citation like [this]_, one 
can also refer to it like this_.

.. [this] here.
"""
        exp_str = """\
Citation references, like CIT2002((A citation (as often used in journals).)). Note that citations may get rearranged, e.g., to the bottom of the "page".

Citation labels contain alphanumerics, underlines, hyphens and fullstops. Case is not significant.

Given a citation like this((here.)), one can also refer to it like <a href="#this">this</a>.

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_external_hyperlink_targets(self):
        str = """\
External hyperlinks, like Python_.

.. _Python: http://www.python.org/
"""
        exp_str = """\
External hyperlinks, like [http://www.python.org/:title=Python].

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_embedded_uris(self):
        str = """\
External hyperlinks, like `Python <http://www.python.org/>`_.
"""
        exp_str = """\
External hyperlinks, like [http://www.python.org/:title=Python].

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_internal_hyperlink_targets(self):
        str = """\
Internal crossreferences, like example_.

.. _example:

This is an example crossreference target.
"""
        exp_str = """\
Internal crossreferences, like <a href="#example">example</a>.

<a name="example"></a>
This is an example crossreference target.

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_indirect_hyperlink_targets(self):
        str = """\
Python_ is `my favourite 
programming language`__.

.. _Python: http://www.python.org/

__ Python_
"""
        exp_str = """\
[http://www.python.org/:title=Python] is [http://www.python.org/:title=my favourite programming language].

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_implicit_hyperlink_targets(self):
        str = """\
Titles are targets, too 
======================= 

Implict references, like `Titles are 
targets, too`_.
"""
        exp_str = """\
*Titles are targets, too

Implict references, like <a href="#titles-are-targets-too">Titles are targets, too</a>.

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_directives(self):
        str = """\
For instance:

.. image:: images/ball1.gif
"""
        exp_str = """\
For instance:

<img src="images/ball1.gif">
"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_substitution_references_and_definitions(self):
        str = """\
The |biohazard| symbol must be used on containers used to dispose of medical waste.

.. |biohazard| image:: biohazard.png
"""
        exp_str = """\
The <img src="biohazard.png" alt="biohazard"> symbol must be used on containers used to dispose of medical waste.

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_comments(self):
        str = """\
Comment
=======

.. This text will not be shown 
   (but, for instance, in HTML might be 
   rendered as an HTML comment)
"""
        exp_str = """\
*Comment

<!--
This text will not be shown (but, for instance, in HTML might be rendered as an HTML comment)
-->

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_comment_block(self):
        str = """\
Comment block
-------------

An "empty comment" does not 
consume following blocks. 
(An empty comment is ".." with 
blank lines before and after.)

..

        So this block is not "lost", 
        despite its indentation.
"""
        exp_str = """\
*Comment block

An "empty comment" does not consume following blocks. (An empty comment is ".." with blank lines before and after.)

<!--

-->

>>
So this block is not "lost", despite its indentation.
<<

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_niconico_directive(self):
        str = """\
Hatena "niconico" notation.

.. niconico:: sm9
"""
        exp_str = """\
Hatena "niconico" notation.

[niconico:sm9]
"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_google_directive(self):
        str = """\
Hatena "google" notation.

.. google:: rst2hatena

.. google:: Guid van Rossum
    :target: image

.. google:: Python
    :target: news
"""
        exp_str = """\
Hatena "google" notation.

[google:rst2hatena]
[google:image:Guid van Rossum]
[google:news:Python]
"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_amazon_directive(self):
        str = """\
Hatena "amazon" notation.

.. amazon:: Python
"""
        exp_str = """\
Hatena "amazon" notation.

[amazon:Python]
"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_wikipedia_directive(self):
        str = """\
Hatena "wikipedia" notation.

.. wikipedia:: Python

.. wikipedia:: Python
    :lang: en
"""
        exp_str = """\
Hatena "wikipedia" notation.

[wikipedia:Python]
[wikipedia:en:Python]
"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_twitter_directive(self):
        str = """\
Hatena "twitter" notation.

.. twitter:: 88453766201880576
    :type: title

.. twitter:: 88453766201880576
    :type: tweet

.. twitter:: 88453766201880576
    :type: detail

.. twitter:: 88453766201880576
    :type: tree

.. twitter:: @naoina
"""
        exp_str = """\
Hatena "twitter" notation.

[twitter:88453766201880576:title]
[twitter:88453766201880576:tweet]
[twitter:88453766201880576:detail]
[twitter:88453766201880576:tree]
[twitter:@naoina]
"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_twitter_roles(self):
        str = """\
roles for Hatena "twitter" notation.

:twitter:`naoina`
:twitter:`@naoina`

before text :twitter:`@naoina` after text
"""
        exp_str = """\
roles for Hatena "twitter" notation.

[twitter:@naoina] [twitter:@naoina]

before text [twitter:@naoina] after text

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_map_directive(self):
        str = """\
Hatena "map" notation.

.. map::
    :x: 139.6983
    :y: 35.6514

.. map::
    :x: 139.6983
    :y: 35.6514
    :type: map

.. map::
    :x: 139.6983
    :y: 35.6514
    :type: satellite
    :height: 300

.. map::
    :x: 139.6983
    :y: 35.6514
    :type: hybrid
    :width: 300
"""
        exp_str = """\
Hatena "map" notation.

map:x139.6983y35.6514
map:x139.6983y35.6514:map
map:x139.6983y35.6514:satellite:h300
map:x139.6983y35.6514:hybrid:w300
"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


    def test_code_blocks(self):
        str = """\
.. code-block:: python

   def main():
       for i in range(1, 101):
           if i % 15 == 0:
              print "FizzBuzz"
           elif i % 5 == 0:
              print "Buzz"
           elif i % 3 == 0:
              print "Fizz"
           else:
              print i

   if __name__ == '__main__':
       main()
"""
        exp_str = """\
>|python|
def main():
    for i in range(1, 101):
        if i % 15 == 0:
           print "FizzBuzz"
        elif i % 5 == 0:
           print "Buzz"
        elif i % 3 == 0:
           print "Fizz"
        else:
           print i

if __name__ == '__main__':
    main()
||<

"""
        output = publish_string(str, writer=self.writer)
        self.assertEqual(exp_str, output)


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(WriterTest)
    try:
        from growltestrunner import GrowlTestRunner
        GrowlTestRunner().run(suite)
    except:
        unittest.main()



# -*- coding:utf-8 -*-

# Copyright (c) 2011 Naoya INADA <naoina@kuune.org>
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.

__author__ = "Naoya INADA <naoina@kuune.org>"

__all__ = [
    "Niconico",
    "Google",
    "Amazon",
    "Wikipedia",
    "Twitter",
    "Map",
]

import re

from docutils.parsers.rst import Directive
from docutils.parsers.rst import directives

from rst2hatena import hatena_nodes


class Niconico(Directive):
    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = {}

    def run(self):
        video_id = directives.unchanged_required(self.arguments[0])
        if not re.match(r"sm\d+?", video_id):
            raise self.error(
                'Error in "%s" directive: "%s" is not a valid video id.'
                % (self.name, video_id))
        niconico_node = hatena_nodes.niconico(self.block_text,
                videoid=video_id)
        return [niconico_node]


class Google(Directive):
    target_values = ("image", "news")

    def target(arg):
        return directives.choice(arg, Google.target_values)

    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = {
        "target": target,
    }

    def run(self):
        self.options["query"] = directives.unchanged_required(
                self.arguments[0])
        google_node = hatena_nodes.google(self.block_text, **self.options)
        return [google_node]


class Amazon(Directive):
    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = {}

    def run(self):
        amazon_query = directives.unchanged_required(self.arguments[0])
        amazon_node = hatena_nodes.amazon(self.block_text, query=amazon_query)
        return [amazon_node]


class Wikipedia(Directive):
    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = {
        "lang": directives.unchanged,
    }

    def run(self):
        self.options["query"] = directives.unchanged_required(
                self.arguments[0])
        wikipedia_node = hatena_nodes.wikipedia(self.block_text,
                **self.options)
        return [wikipedia_node]


class Twitter(Directive):
    quotetype_values = ("title", "tweet", "detail", "tree")

    def quote_type(arg):
        return directives.choice(arg, Twitter.quotetype_values)

    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = {
        "type": quote_type,
    }

    def run(self):
        id = directives.unchanged_required(self.arguments[0])
        if id.startswith("@"):
            if "type" in self.options:
                raise self.error(
                    'Error in "%s" directive: "type" option must not be given '
                    'if argument is Twitter ID(starts with "@").'
                    % (self.name))
        elif "type" not in self.options:
            raise self.error(
                'Error in "%s" directive: "type" option must be given '
                'if argument is Tweet ID.' % (self.name))
        self.options["id"] = id
        twitter_node = hatena_nodes.twitter(self.block_text, **self.options)
        return [twitter_node]


class Map(Directive):
    displaytype_values = ("map", "satellite", "hybrid")

    def display_type(arg):
        return directives.choice(arg, Map.displaytype_values)

    required_arguments = 0
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = {
        "x": directives.unchanged_required,
        "y": directives.unchanged_required,
        "width": directives.positive_int,
        "height": directives.positive_int,
        "type": display_type,
    }

    def run(self):
        if "x" not in self.options or "y" not in self.options:
            raise self.error(
                'Error in "%s" directive: "x" and "y" option is required.'
                % (self.name))
        if "width" in self.options and "height" in self.options:
            raise self.error(
                'Error in "%s" directive: "width" and "height" option can not '
                'be combined.'
                % (self.name))
        map_node = hatena_nodes.map(self.block_text, **self.options)
        return [map_node]



from setuptools import setup, find_packages
import sys, os

version = '0.2.1'

setup(name='rst2hatena',
      version=version,
      description="Convert reStructuredText to Hatena Diary syntax",
      classifiers=[
          "Development Status :: 4 - Beta",
          "Environment :: Console",
          "Operating System :: OS Independent",
          "License :: OSI Approved :: BSD License",
          "Programming Language :: Python",
          "Topic :: Documentation",
          "Topic :: Software Development :: Libraries :: Python Modules",
          "Topic :: Text Processing",
          "Topic :: Utilities",
      ], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='hatena restructuredtext',
      author='Yosuke Ikeda',
      author_email='alpha.echo.35@gmail.com',
      url='http://bitbucket.org/ae35/rst2hatena',
      license='BSD',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          "docutils",
      ],
      entry_points={'console_scripts': ['rst2hatena = rst2hatena.rst2hatena:main']},
      test_suite="rst2hatena.tests"
)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""Hatena document writer
"""
from docutils import writers, nodes

from base_translator import BaseTranslator


class Writer(writers.UnfilteredWriter):

    supported = ('hatena',)
    """Formats this writer supports."""

    config_section = 'hatena writer'
    config_section_dependencies = ('writers',)

    def translate(self):
        visitor = HatenaTranslator(self.document)
        self.document.walkabout(visitor)
#        print self.document.pformat()
        self.output = visitor.astext()


class HatenaTranslator(BaseTranslator):
    """
    """
    def __init__(self, document):
        BaseTranslator.__init__(self, document)

        self.section_level = 0
        self.initial_header_level = 1  # TODO: int(settings.initial_header_level)
        self.line_block_level = 0
        self.list_level = 0
        self.is_enum = True
        self.block_quote_level = 0

        self.in_reference = None
        self.in_internal_reference = None
        self.in_line = None
        self.in_definition = None
        self.in_paragraph = None
        self.in_thead = None
        self.in_comment = None
        self.in_author = None

        self.footnote_idx_dict = {}
        self.footnote_idx = 0
        self.footnote_ids = None

        self.in_citation_key = None
        self.citation_idx_dict = {}
        self.citation_idx = 0
        self.citation_ids = None

    def astext(self):
        return ''.join(self.body)

    def visit_Text(self, node):
        text = node.astext()
        if self.in_definition or self.in_line or self.in_comment \
                or self.in_paragraph or self.in_author:
            text = ' '.join(text.split('\n'))
        self.body.append(text)

    def depart_Text(self, node):
        pass

    # ================
    #  Title Elements
    # ================

    def visit_title(self, node):
        if isinstance(node.parent, nodes.topic):
            self.body.append(':')
        elif isinstance(node.parent, nodes.document):
            self.body.append('*' * self.initial_header_level)
        else:
            h_level = self.section_level + self.initial_header_level
            if h_level > 3:
                h_level = 3
            self.body.append('*' * h_level)

    def depart_title(self, node):
        if isinstance(node.parent, nodes.topic):
            self.body.append(':')
            return
        self.body.append('\n')
        self.body.append('\n')

    def visit_subtitle(self, node):
        if isinstance(node.parent, nodes.document):
            self.body.append('*' * (self.initial_header_level + 1))
        elif isinstance(node.parent, nodes.section):
            h_level = self.section_level + self.initial_header_level
            if h_level > 3:
                h_level = 3
            self.body.append('*' * h_level)

    def depart_subtitle(self, node):
        self.body.append('\n')
        self.body.append('\n')

    # ========================
    #  Bibliographic Elements
    # ========================
    def visit_author(self, node):
        self.in_author = 1
        self.body.append(':Author:')

    def depart_author(self, node):
        self.in_author = None
        self.body.append('\n')

    def visit_organization(self, node):
        self.body.append(':Organization:')

    def depart_organization(self, node):
        self.body.append('\n')

    def visit_address(self, node):
        self.body.append(':Address:')

    def depart_address(self, node):
        self.body.append('\n')

    def visit_contact(self, node):
        self.body.append(':Contact:')

    def depart_contact(self, node):
        self.body.append('\n')

    def visit_version(self, node):
        self.body.append(':Version:')

    def depart_version(self, node):
        self.body.append('\n')

    def visit_revision(self, node):
        self.body.append(':Revision:')

    def depart_revision(self, node):
        self.body.append('\n')

    def visit_status(self, node):
        self.body.append(':Status:')

    def depart_status(self, node):
        self.body.append('\n')

    def visit_date(self, node):
        self.body.append(':Date:')

    def depart_date(self, node):
        self.body.append('\n')

    def visit_copyright(self, node):
        self.body.append(':Copyright:')

    def depart_copyright(self, node):
        self.body.append('\n')

    # =====================
    #  Structural Elements
    # =====================

    def visit_section(self, node):
        self.section_level += 1

    def depart_section(self, node):
        self.section_level -= 1

    def visit_topic(self, node):
        self.in_definition = 1

    def depart_topic(self, node):
        self.in_definition = None

    def visit_transition(self, node):
        self.body.append('<hr>')

    def depart_transition(self, node):
        self.body.append('\n')
        self.body.append('\n')

    # ===============
    #  Body Elements
    # ===============

    def visit_paragraph(self, node):
        self.in_paragraph = 1

    def depart_paragraph(self, node):
        self.in_paragraph = None
        if isinstance(node.parent, nodes.entry) or self.footnote_idx != 0 or self.citation_idx != 0:
            return
        self.body.append('\n')
        if not isinstance(node.parent, nodes.list_item) \
                and not isinstance(node.parent, nodes.system_message) \
                and not isinstance(node.parent, nodes.topic) \
                and not isinstance(node.parent, nodes.block_quote) and self.block_quote_level != 1:
            self.body.append('\n')

    def visit_bullet_list(self, node):
        self.is_enum = False
        self.list_level += 1

    def depart_bullet_list(self, node):
        self.list_level -= 1
        if not isinstance(node.parent, nodes.list_item) \
                and not isinstance(node.parent, nodes.entry):
            self.body.append('\n')

    def visit_enumerated_list(self, node):
        self.is_enum = True
        self.list_level += 1

    def depart_enumerated_list(self, node):
        self.list_level -= 1
        if not isinstance(node.parent, nodes.list_item) \
                and not isinstance(node.parent, nodes.entry):
            self.body.append('\n')

    def visit_list_item(self, node):
        if self.is_enum:
            li = '+'
        else:
            li = '-'
        self.body.append(li * (self.list_level))
        self.body.append(' ')

    def depart_list_item(self, node):
        pass

    def visit_term(self, node):
        self.body.append(':')

    def depart_term(self, node):
        self.body.append(':')

    def visit_definition(self, node):
        self.in_definition = 1

    def depart_definition(self, node):
        self.in_definition = None
        pass

    def visit_field_name(self, node):
        self.body.append(':')

    def depart_field_name(self, node):
        self.body.append(':')

    def visit_option(self, node):
        self.body.append(':')

    def depart_option(self, node):
        self.body.append(':')

    def visit_option_argument(self, node):
        if node.has_key('delimiter'):
            self.body.append(node['delimiter'])

    def depart_option_argument(self, node):
        pass

    def visit_option_list_item(self, node):
        self.in_definition = 1

    def depart_option_list_item(self, node):
        self.in_definition = None

    def visit_literal_block(self, node):
        self.body.append('>||')
        self.body.append('\n')

    def depart_literal_block(self, node):
        self.body.append('\n')
        self.body.append('||<')
        self.body.append('\n')
        self.body.append('\n')

    def visit_line_block(self, node):
        if self.line_block_level == 0:
            self.body.append('>|')
            self.body.append('\n')
        self.line_block_level += 1

    def depart_line_block(self, node):
        self.line_block_level -= 1
        if self.line_block_level == 0:
            self.body.append('|<')
            self.body.append('\n')
            self.body.append('\n')

    def visit_line(self, node):
        self.in_line = 1
        self.body.append('    ' * (self.line_block_level - 1))

    def depart_line(self, node):
        self.in_line = None
        self.body.append('\n')

    def visit_doctest_block(self, node):
        self.body.append('>|python|')
        self.body.append('\n')

    def depart_doctest_block(self, node):
        self.body.append('\n')
        self.body.append('||<')
        self.body.append('\n')
        self.body.append('\n')

    def visit_block_quote(self, node):
        self.block_quote_level += 1
        if self.block_quote_level == 1:
            self.body.append('>>')
        self.body.append('\n')
        self.body.append('    ' * (self.block_quote_level - 1))

    def depart_block_quote(self, node):
        self.block_quote_level -= 1
        if self.block_quote_level == 0:
            self.body.append('<<')
            self.body.append('\n')
            self.body.append('\n')

    def visit_comment(self, node):
        self.in_comment = 1
        self.body.append('<!--\n')

    def depart_comment(self, node):
        self.in_comment = None
        self.body.append('\n')
        self.body.append('-->\n')
        self.body.append('\n')

    def visit_substitution_definition(self, node):
        raise nodes.SkipNode

    def depart_substitution_definition(self, node):
        pass

    def visit_target(self, node):
        if node.has_key('refid'):
            self.body.append('<a name="%s"></a>' % node['refid'])
            self.body.append('\n')

    def depart_target(self, node):
        pass

    def visit_thead(self, node):
        self.in_thead = 1

    def depart_thead(self, node):
        self.in_thead = None

    def visit_tbody(self, node):
        pass

    def depart_tbody(self, node):
        self.body.append('\n')

    def visit_row(self, node):
        pass

    def depart_row(self, node):
        self.body.append('|')
        self.body.append('\n')

    def visit_entry(self, node):
        self.body.append('|')
        if self.in_thead:
            self.body.append('*')

    def depart_entry(self, node):
        pass

    def visit_footnote(self, node):
        if not node.has_key('ids'):
            raise nodes.SkipNode
        self.footnote_ids = node['ids'][0]
        if not self.footnote_idx_dict.has_key(self.footnote_ids):
            raise nodes.SkipNode
        self.footnote_idx = len(self.body)

    def depart_footnote(self, node):
        footnote = self.body[self.footnote_idx:]
        for idx in self.footnote_idx_dict[self.footnote_ids]:
            self.body[idx] = ''.join(footnote)
        del self.body[self.footnote_idx:]
        self.footnote_idx = 0

    def visit_citation(self, node):
        if not node.has_key('ids'):
            raise nodes.SkipNode
        self.citation_ids = node['ids'][0]
        if not self.citation_idx_dict.has_key(self.citation_ids):
            raise nodes.SkipNode
        self.citation_idx = len(self.body)

    def depart_citation(self, node):
        citation = self.body[self.citation_idx:]
        for idx in self.citation_idx_dict[self.citation_ids]:
            self.body[idx] = ''.join(citation)
        del self.body[self.citation_idx:]
        self.citation_idx = 0

    def visit_label(self, node):
        raise nodes.SkipNode

    def visit_system_message(self, node):
        self.body.append('<!--\n')

    def depart_system_message(self, node):
        self.body.append('-->\n')
        self.body.append('\n')

    # =================
    #  Inline Elements
    # =================

    def visit_emphasis(self, node):
        self.body.append('<span style="font-style:italic;">')

    def depart_emphasis(self, node):
        self.body.append('</span>')

    def visit_strong(self, node):
        self.body.append('<span style="font-weight:bold;">')

    def depart_strong(self, node):
        self.body.append('</span>')

    def visit_literal(self, node):
        self.body.append('<span style="font-style:italic;">')

    def depart_literal(self, node):
        self.body.append('</span>')

    def visit_reference(self, node):
        if node.has_key('refuri'):
            self.in_reference = 1
            href = node['refuri']
            self.body.append('[')
            self.body.append(href)
            self.body.append(':title=')
        elif node.has_key('refid'):
            self.in_internal_reference = 1
            self.body.append('<a href="#%s">' % node['refid'])

    def depart_reference(self, node):
        if self.in_reference:
            self.body.append(']')
        if self.in_internal_reference:
            self.body.append('</a>')
        if not isinstance(node.parent, nodes.TextElement):
            self.body.append('\n')
        self.in_reference = None
        self.in_internal_reference = None

    def visit_title_reference(self, node):
        self.body.append('<span style="font-style:italic;">')

    def depart_title_reference(self, node):
        self.body.append('</span>')

    def visit_footnote_reference(self, node):
        if not node.has_key('refid'):
            raise nodes.SkipNode
        key = node['refid']
        self.body.append('((')
        if not self.footnote_idx_dict.has_key(key):
            self.footnote_idx_dict[key] = []
        self.footnote_idx_dict[key].append(len(self.body))
        self.body.append('')  # dummy for replace later
        self.body.append('))')
        raise nodes.SkipNode

    def depart_footnote_reference(self, node):
        pass

    def visit_citation_reference(self, node):
        if not node.has_key('refid'):
            return
        self.in_citation_key = node['refid']

    def depart_citation_reference(self, node):
        self.body.append('((')
        if not self.citation_idx_dict.has_key(self.in_citation_key):
            self.citation_idx_dict[self.in_citation_key] = []
        self.citation_idx_dict[self.in_citation_key].append(len(self.body))
        self.body.append('')  # dummy footnote for replace later
        self.body.append('))')
        self.in_citation_key = None

    def visit_image(self, node):
        if node.has_key('uri'):
            self.body.append('<img src="%s"' % node['uri'])
            if node.has_key('alt'):
                self.body.append(' alt="%s"' % node['alt'])
            self.body.append('>')

    def depart_image(self, node):
        if not self.in_paragraph:
            self.body.append('\n')

    def visit_code_block(self, node):
        typ = node['type']
        self.body.append('>|%s|' % typ)
        self.body.append('\n')

    def depart_code_block(self, node):
        self.body.append('\n')
        self.body.append('||<')
        self.body.append('\n')
        self.body.append('\n')

    # ==================
    #  Hatena directives
    # ==================

    def visit_niconico(self, node):
        self.body.append("[niconico:%s]" % node["videoid"])

    def depart_niconico(self, node):
        self.body.append("\n")

    def visit_google(self, node):
        target = node["target"] + ":" if "target" in node else ""
        self.body.append("[google:%s]" % (target + node["query"]))

    def depart_google(self, node):
        if not self.in_paragraph:
            self.body.append("\n")

    def visit_amazon(self, node):
        self.body.append("[amazon:%s]" % node["query"])

    def depart_amazon(self, node):
        if not self.in_paragraph:
            self.body.append("\n")

    def visit_wikipedia(self, node):
        lang = node["lang"] + ":" if "lang" in node else ""
        self.body.append("[wikipedia:%s]" % (lang + node["query"]))

    def depart_wikipedia(self, node):
        if not self.in_paragraph:
            self.body.append("\n")

    def visit_twitter(self, node):
        type = ":" + node["type"] if "type" in node else ""
        self.body.append("[twitter:%s]" % (node["id"] + type))

    def depart_twitter(self, node):
        if not self.in_paragraph:
            self.body.append("\n")

    def visit_map(self, node):
        values = ["map", "x%sy%s" % (node["x"], node["y"])]
        if "type" in node:
            values.append(node["type"])
        if "width" in node:
            values.append("w%s" % (node["width"]))
        elif "height" in node:
            values.append("h%s" % (node["height"]))
        self.body.append(":".join(values))

    def depart_map(self, node):
        if not self.in_paragraph:
            self.body.append("\n")



# -*- coding: utf-8 -*-

# Copyright (c) 2011 Naoya INADA <naoina@kuune.org>
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.

__author__ = "Naoya INADA <naoina@kuune.org>"

from docutils import nodes
from docutils import utils
from docutils.parsers.rst import roles
from docutils.parsers.rst.languages import en


def twitter_role(name, rawtext, text, lineno, inliner, options={}, content=[]):
    text = utils.unescape(text)
    if text[0] != '@':
        text = '@' + text
    node = nodes.raw(rawtext, "[twitter:%s]" % text)
    return ([node], [])

_hatena_roles = (
    ('twitter', twitter_role),)

for name, role in _hatena_roles:
    roles.register_canonical_role(name, role)
    en.roles[name] = name
